/*
   ----------------------------------------------------------------------------				
   SQLCustomButtons.cc
	Programmer: Ken Mayer
   Date......: May, 2015
   Notes.....: Custom Pushbuttons for the Invoice example in
	            the book: The dBASE Book Plus
					
					Based heavily on the dUFLP (CustButt.cc), hence you 
					will see below the #include statements for the different
					language files, which will be incorporated at some point.
					
					These buttons are designed to work with ADO databases
					specifically which means we are not using the 
					dataLink property for controls, and the record buffering
					of the rowset object. Some methods and events
					for rowsets should work fine, however -- navigation,
					for example. 
					
					The Navigation Buttons will rely on the use of the form's
					onNavigate() event handler to re-load the data controls
					with the data from the table. 
					
					The form is always in an "edit" mode, so we don't need
					to worry about that functionality, but any code that
					relies on the rowset's state property is not going to
					be very useful.
					
					We will be relying on code in the custom form class
					UILayer.cfm, and the Data Module DataLayer.dmd to
					do some of what is needed.
	----------------------------------------------------------------------------				
*/

// See comments below regarding translations ...:
#include <custButtEnglish.h>    // completed
//#include <custButtItalian.h>    // completed
//#include <custButtGerman.h>     // completed
//#include <custButtDanish.h>     // completed
//#include <custButtDutch.h>      // completed
//#include <custButtNorwegian.h>  // completed
//#include <custButtFrench.h>     // completed
//#include <custButtCatalan.h>    // completed
//#include <custButtSpanish.h>    // completed
//#include <custButtPortuguese.h> // completed
//#include <custButtTurkish.h>    // completed
//#include <custButtSwedish.h>    // completed


#DEFINE ButtonWidth 64
#DEFINE ButtonHeight 24

/*
   To start, these buttons will be text-based,
	in a later chapter we will change them to 
	use images from the GlyphX image pack that
	is installed with dBASE ...
*/
class BaseButton( oParent ) of Pushbutton( oParent )
   with( this )
      text     := "Base Class"
		metric   := 6 // Pixels
		width    := ButtonWidth
		height   := ButtonHeight
		speedBar := true // can't get focus
   endwith
	// Custom Property:
	this.rowset = null // empty

   function getRowset()
      local rRowToActOn
      rRowToActOn = null // default
      // check to see if rowset property has been set -- allow
		// developer to reference a *specific* rowset rather than
		// the default form.rowset:
      if this.rowset # null
         rRowToActOn = this.rowset
        // ok, this.rowset has not been set, check to see
        // if the parent has a rowset that's valid
        // (the parent will be "NavigateButtons", a container,
        // or the form):
      elseif type( "this.parent.rowset" ) == "O" and;
             this.parent.rowset # null
         rRowToActOn = this.parent.rowset
      // otherwise we check for the form's rowset:
      elseif form.rowset # null
         rRowToActOn = form.rowset
      endif // this.rowset # null
   return rRowToActOn	
endclass

class TopButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "First"
   endwith

   function onClick
	   local rRowToActOn, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Navigate with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( "Save changes before leaving record?", ;
								"Data has changed",32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// navigate to first record
         rRowToActOn.first()
      endif
		// the UILayer.cfm file uses the form's onNavigate
		// event handler, and will refresh the screen ...
	return
endclass

class PreviousButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Previous"
   endwith

   function onClick
      local rRowToActOn, nState, nAnswer
      rRowToActOn = null

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Navigate with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( "Save changes before leaving record?", ;
								"Data has changed",32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// attempt to navigate:
         if ( not rRowToActOn.next(-1) )
            msgbox( "At first row in rowset", ;
                    "Can't Navigate!", 64 )
            rRowToActOn.next() // back off end of rowset marker
         endif // navigate
      endif // rRowToActOn
	return
endclass

class NextButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Next"
   endwith

   function onClick
      local rRowToActOn, nState, nAnswer
      rRowToActOn = null

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Navigate with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( "Save changes before leaving record?", ;
								"Data has changed",32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// attempt to navigate:
         if ( not rRowToActOn.next() )
            msgbox( "At last row in rowset", ;
                    "Can't Navigate!", 64 )
            rRowToActOn.next(-1) // back off end of rowset marker
         endif // attempt to navigate
      endif // form.modified
	return
endclass

class BottomButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Last"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Navigate with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( "Save changes before leaving record?", ;
								"Data has changed",32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
         // to last row in rowset:		
         rRowToActOn.last()
      endif
	return
endclass

class SaveButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Save"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Save with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
				return // nothing to save
			endif
			// call SaveRecord() method
			if form.modified
            form.SaveRecord()
			endif // form.modified
      endif // we have a valid rowset
	return // end of method
endclass

class AbandonButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Abandon"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Abandon with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
				return // nothing to save
			endif
			// call AbandonRecord() method
			if form.modified
            form.AbandonRecord()
			endif // form.modified
      endif
	return
endclass

class AppendButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Append"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Append with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			// as the AppendRecord method clears out the controls,
			// check to see if the user wants to save any changes:
			if form.modified
				if msgbox( "Save changes before leaving record?", ;
								"Data has changed",32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// call AppendRecord() method
         form.AppendRecord()
      endif
	return
endclass

class DeleteButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Delete"
   endwith
	// necessary for determining if a cascade
	// delete will occur. When placing this 
	// button on the form, in the form's
	// onOpen event handler, or overridden
	// Open method, these values should
	// be set:
	this.cascade         = false
	this.childTable      = null
	this.foreignKeyField = null // link to parent table
	
   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( "Cannot Delete with no rowset!",;
                 "Can't do it!", 16 )
         return
      else
		   // ask if they really want to delete the record:
		   if msgbox( "Are you sure you wish to delete this record?",;
		              "Delete Record?", 32+4 ) == 6 // Yes
   			// call DeleteRecord() method
            form.DeleteRecord( this.cascade, this.childTable, ;
				                   this.foreignKeyField )
			endif // msgbox()
      endif // row ...
	return
endclass

class CloseButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Close"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
		   // if not, then just close the form
         form.close()
      endif
		
	   // before closing the form, check to see if the
		// record is being modified (includes
		// appending -- if the user slipped into
		// append mode, but hasn't changed anything,
		// the form's modified property will be
		// false) -- and if so, ask if they 
		// want to save changes before closing:
		if form.modified
			if msgbox( "Save changes before leaving record?", ;
                     "Data has changed",32+4) == 6 // Yes
            form.SaveRecord()
			endif // msgbox()
		endif // form.modified
      // close the form:
		form.close()
	return
endclass
