/*
   ----------------------------------------------------------------------------
   BusinessLayer.cc
	Programmer: Ken Mayer
   Date......: June, 2015
   Notes.....: Business Layer for sample Invoice system for
	            The dBASE Book Plus.
					
					The idea is to put any data validation code here.
               The difficulty is in setting up data validation
					(The "Business Layer") as a middle step between 
					the User Interface and Data layers.
					
					At this time, the goal is to do most of this as
					"form level" validation, meaning each form should
					have its own rules. For example, a Customers form
					might have a requirement that some fields cannot be
					left blank, while others can. You would need to check
					for that.
					
					One way to do this is to override the Save Button's
					onClick event handler in the form, so that it calls
					code here, which would validate the data and 
					return a true or false value ...
					
					If the form's fields are okay, then we can move on,
					otherwise the user cannot save the record.
					
					This might look like:
					   function SaveButton1_onClick()
						   set procedure to BusinessLayer.cc
							local bOK, oValidate
							// parameter must be the controls
							// array on the form:
							oValidate = new CustValidate( form.aSQLControls )
							bOK = oValidate.isValid()
							release object oValidate
							oValidate := null
							close procedure BusinessLayer.cc
							
							if not bOK
							   // msgbox?
								return
							endif
							
							// procede to save:
							SaveButton::OnClick()
						return
						
               By its very nature, Data Validation is application
					specific. 

   Classes: CustValidate
            EMailAddress -- copied out of dUFLP to avoid issues
                            of someone not having the dUFLP installed
                            properly on their machine ... validates
                            an email address
	// Others are left to the reader to try to work out ...
	----------------------------------------------------------------------------				
*/

#DEFINE HIGHLIGHT       "WindowText/0x80ffff" // yellow background
#DEFINE NORMAL          "WindowText/Window"
#DEFINE ERRORCOLOR      "WHITE/RED"

class CustValidate // Customer Validation
   /*
	   Customer Form Validation -- check to ensure that the customers
		table has any required data, set error flag for controls
		that have invalid data. The only things we have to worry about
	   for customer data is empty fields ... we need the business name,
		basic address (address2 field is optional) ... we don't need
		the email address, although it is handy.
      
      This class has a method for each field that we need validation
      for. It allows the code to be called individually if needed,
      or to call the "FormLevel" method below.
      
      Examples:
      
         set procedure to BusinessLayer.cc
         oCustValid = new CustValidate()
         bOK = oCustValid.FormLevel()
         if not bOK
            // do some processing ...
         else
            // valid, do other processing
         endif
         
         // or -- check for individual field:
         set procedure to BusinessLayer.cc
         oCustValid = new CustValidate()
         if not oCustValid.BusNameValid( cBusName )
            // do some processing
         endif
	*/
   
	function FormLevel
	   parameters aControls
		// These errors should only ever be seen by the developer
		// while testing ...:
		if pCount() < 1
			msgbox( "Validation requires a reference to "+chr(13)+;
					  "the form's aSQLControls array.",;
					  "CustValidate Error", 16 )
			return
		endif

		if type( "aControls" ) # "A"
			msgbox( "Parameter for validation requires a "+chr(13)+;
					  "refrence to the form's aSQLControls array.",;
					  "CustValidate Error", 16 )
			return
		endif

		// make reference a custom property of the class:
		this.Controls = aControls
      
		// return value, set to assume all is okay:
		local bCtrlValid, bFormValid
		bFormValid = true 
		
		// call individual methods as needed:
		// 1: Customer Number -- no validation required
      
		// 2: Business Name:
		bCtrlValid = class::BusNameValid( this.Controls[ 2 ].value )
      class::SetErrorColors( this.Controls[ 2 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 2 ].error := true
		   bFormValid := false
      else
         this.Controls[ 2 ].error := false
		endif
		
		// 3: Address 1
		bCtrlValid = class::Address1Valid( this.Controls[ 3 ].value )
      class::SetErrorColors( this.Controls[ 3 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 3 ].error := true
		   bFormValid := false
      else
         this.Controls[ 3 ].error := false
		endif

      // 4: Address 2
		// if this has a value and Address1 is blank 
		// we have an error
      bCtrlValid = class::Address2Valid( this.Controls[ 4 ].value,;
                                         this.Controls[ 3 ].value )
      class::SetErrorColors( this.Controls[ 4 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 4 ].error := true
		   bFormValid := false
      else
         this.Controls[ 4 ].error := false
		endif
		
		// 5: City
		bCtrlValid = class::CityValid( this.Controls[ 5 ].value )
      class::SetErrorColors( this.Controls[ 5 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 5 ].error := true
		   bFormValid := false
      else
         this.Controls[ 5 ].error := false
		endif
		
		// 6: State
		bCtrlValid = class::StateValid( this.Controls[ 6 ].value )
      class::SetErrorColors( this.Controls[ 6 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 6 ].error := true
		   bFormValid := false
      else
         this.Controls[ 6 ].error := false
		endif

      // 7: Postal Code
		bCtrlValid = class::PostalCodeValid( this.Controls[ 7 ].value )
      class::SetErrorColors( this.Controls[ 7 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 7 ].error := true
		   bFormValid := false
      else
         this.Controls[ 7 ].error := false
		endif
		
		// 8: EMail Address
		bCtrlValid = class::EMailValid( this.Controls[ 8 ].value )
      class::SetErrorColors( this.Controls[ 8 ], bCtrlValid )
      if not bCtrlValid
         this.Controls[ 8 ].error := true
		   bFormValid := false
      else
         this.Controls[ 8 ].error := false
		endif
      
	return bFormValid
   // End of method: FormLevel
	
	function BusNameValid
	   // validate the Business Name field
		// Only requirement is that it not be empty
	   parameters cBusName
		local bOK
		bOK = true
		if empty( cBusName ) 
         bOK = false
		endif
	return bOK
	
	function Address1Valid
	   // validate the Address1 field
		// Only requirement is that it not be empty
	   parameters cAddress1
		local bOK
		bOK = true
		if empty( cAddress1 ) 
         bOK = false
		endif
	return bOK
   
	function Address2Valid
	   // validate the Address2 field
		// If not empty, and address 1 is empty
      // we have an error. Parameters
      // passed in this sequence because
      // primary check is Address2 ...:
	   parameters cAddress2, cAddress1
		local bOK
		bOK = true
		if NOT empty( cAddress2 ) and empty( cAddress1 )
         bOK = false
		endif
	return bOK
   
	function CityValid
	   // validate the City field
		// Only requirement is that it not be empty
	   parameters cCity
		local bOK
		bOK = true
		if empty( cCity ) 
         bOK = false
		endif
	return bOK
   
   function StateValid
	   // validate the State field
		// Only requirement is that it not be empty
	   parameters cState
		local bOK
		bOK = true
		if empty( cState ) 
         bOK = false
		endif
	return bOK
   
   function PostalCodeValid
	   // validate the PostalCode field
		// Only requirement is that it not be empty
	   parameters cPostalCode
		local bOK
		bOK = true
		if empty( cPostalCode ) 
         bOK = false
		endif
	return bOK
   
   function EmailValid
	   // validate the Email Address field
		// This checks for an empty email address, 
      // which is actually okay, but if NOT empty,
      // we check to make sure it is a valid 
      // address
	   parameters cEmail
		local bOK, oEmail
		bOK = true
		if not empty( cEmail )
         oEmail = new EmailAddress()
         bOK = oEmail.isValid( cEmail )
		endif
	return bOK

	function SetErrorColors
      // Set control colors
      parameter oControl, bOK
      if bOK
		   // reset colors:
			oControl.colorNormal    := NORMAL
			oControl.colorHighlight := HIGHLIGHT
		else
			// set error color:
			oControl.colorNormal    := ERRORCOLOR
			oControl.colorHighlight := ERRORCOLOR
      endif
   return
   
endclass
// End of class: CustValidate

// ****************************************************************************
//            The following was copied out of the dUFLP for use with 
//            validating email addresses, and to avoid issues with
//            someone using this code not having the dUFLP set up ...
// *****************************************************************************
/*
   ----------------------------------------------------------------------------
    EMailAddress.cc
    Bowen Moursund
    Posted on the dBASE newsgroups, July 6, 2005

    This (non-visual) class attempts to validate an email address.

    	Update: 07.05.2006
      Winfried Schneller
      	New  
	      #define MEHR_ALS_ZWEI_LETTER_TLDS
         check more then 2 letters in TLD
         min 2 letters in TLD

    Usage:
       set procedure to :dUFLP:EMailAddress.cc additive
       oEMail = new EMailAddress()
       // check to see if valid:
       if not oEMail.isValid( <emailaddress> )
          msgbox( oEMail.ErrorMessage, "EMail Address Invalid", 16 )
          return
       endif
       // continue processing ...
   ----------------------------------------------------------------------------
*/

// if a character is valid in an email address and not shown here, add to the end of the list:
#define VALID_CHARS "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@-_.'"
// Top-Level Domains -- with four letters -- add your own to this list with 
// a "|" symbol between, include the dot ("."), if you need a four-letter domain
// name to be considered valid (they are not common).
#define FOUR_LETTER_TLDS ".name|.info|.fake"

// Author Winfried Schneller
// Added new constant for additional check.
// Generally top level domains for countries consist of two characters.
// Now top level domains wider than two characters are checked also.
// Checking Top Level Domains with a length greater than two characters
// "More than two letter TLDs"
#define MEHR_ALS_ZWEI_LETTER_TLDS ".aero|.arpa|.biz|.com|.coop|.edu|.gov|.info|.int|.jobs|.mil|.mobi|.museum|.name|.net|.org|.pro|.travel"

// quelle 
//http://de.wikipedia.org/wiki/Top_Level_Domain
/*
.aero  aeronautics, fr in der Luftfahrt ttige Organisationen  weltweit 
.arpa  TLD des ursprnglichen Arpanets, jetzt verwendet als Address and Routing Parameter Area 
.biz  business, nur fr Firmen  weltweit 
.com  commercial, ursprnglich nur fr US-Firmen, jetzt frei fr jeden  weltweit (*) 
.coop  cooperatives (Genossenschaften) - weltweit 
.edu  educational, nur fr Bildungseinrichtungen (zum Beispiel www.mit.edu) (*) 
.gov  government, nur Regierungsorgane der USA (zum Beispiel www.whitehouse.gov) (*) 
.info  Informationsanbieter  weltweit 
.int  Internationale Regierungsorganisationen (zum Beispiel www.nato.int oder www.eu.int) 
.jobs - jobs, nur fr Firmen mit Stellenangeboten - weltweit 
.mil  military, nur fr militrische Einrichtungen der USA (zum Beispiel www.army.mil) (*) 
.mobi - mobile, Darstellung der Webseiten speziell fr mobile Endgerte, sTLD - weltweit 
.museum  Museen  weltweit 
.name  nur fr natrliche Personen oder Familien (Privatpersonen)  weltweit 
.net  Netzverwaltungseinrichtung  weltweit (*) 
.org  organization, ursprnglich fr nichtkommerzielle Organisationen reserviert, inzwischen allgemein freigegeben  weltweit (*) 
.pro  professions (Anwlte, Steuerberater, rzte, Ingenieure)  nur fr genannte Berufsgruppen der USA, Kanadas, Deutschlands und des Vereinigten Knigreichs 
.travel  fr die Reise-Industrie (z. B. Reisebros, Fluggesellschaften etc.) 
*/

class EmailAddress

  this.ErrorMessage = ""
  this.Address = ""

   function isValid( cAddress )
      if not empty( cAddress )
         this.Address = ltrim(trim( cAddress ))
         cAddress = this.Address
      else
         this.Address = ""
         cAddress = ""
      endif
      this.ErrorMessage = ""

      local bIsValid
      bIsValid = true

      // chk length
      if len( cAddress ) < 5
         // a@b.c should be the shortest an
         // address could be
         bIsValid := false
         this.ErrorMessage := "The email address is too short to be a valid address."

      // chk format
      // has at least one "@"
      elseif not "@" $ cAddress
         bIsValid := false
         this.ErrorMessage := "The email address does not contain the '@' character."

      // has at least one "."
      elseif not "." $ cAddress
         bIsValid := false
         this.ErrorMessage := "The email address does not contain the '.' character."

/*		OLD CHECK
      // has no more than 3 chars after last "."
      elseif len(cAddress)-rat(".", cAddress) > 3 and ;
             not (lower(substr(cAddress, rat(".",cAddress))) $ FOUR_LETTER_TLDS)
         bIsValid := false
         this.ErrorMessage := "The top level domain of the email address is unknown or of an invalid length, or you may have entered more than one email address."
*/

		//New WS 07.05.2006
      // has no 2 chars after last "."
      elseif len(cAddress)-rat(".", cAddress) < 2 
         bIsValid := false
         this.ErrorMessage := "The top level domain of the email address is unknown or of an invalid length, or you may have entered more than one email address."

		//New WS 07.05.2006
      // has no more than 2 chars after last "."
      elseif len(cAddress)-rat(".", cAddress) > 2 and ;
             not (lower(substr(cAddress, rat(".",cAddress))) $ MEHR_ALS_ZWEI_LETTER_TLDS)
         bIsValid := false
         this.ErrorMessage := "The top level domain of the email address is unknown or of an invalid length, or you may have entered more than one email address."


      // has no "_" after the "@"
      elseif rat( "_", cAddress ) > rat( "@", cAddress )
         bIsValid := false
         this.ErrorMessage := "The email address contains a '_' character after the '@'."

      // has only one "@"
      elseif at( "@", cAddress, 2 ) > 0
         bIsValid := false
         this.ErrorMessage := "The email address contains more than one '@' character."

      // has at least one character before the @
      elseif left( cAddress, 1 ) = "@"
         bIsValid := false
         this.ErrorMessage := "The email address does not specify a recipient - there are no characters before the '@'."

      else // chk each char for validity
         for i = 1 to len( cAddress )
             if not substr( cAddress, i, 1 ) $ VALID_CHARS
                bIsValid := false
                this.ErrorMessage := "The email address contains an invalid character: '" + substr( cAddress, i, 1 ) + "'"
                exit
             endif
         next

      endif

   return bIsValid
   // eom: isValid()
endclass

*/
// ----------------------------------------------------------------------------
// End of file: BusinessLayer.cc

