/*
   ----------------------------------------------------------------------------				
   SQLCustomButtons.cc
	Programmer: Ken Mayer
   Date......: May, 2015
   Notes.....: Custom Pushbuttons for the Invoice example in
	            the book: The dBASE Book Plus
					This is the version in Chapter 6
					
					Based heavily on the dUFLP (CustButt.cc), hence you 
					will see below the #include statements for the different
					language files.
					
					These buttons are designed to work with ADO databases
					specifically which means we are not using the 
					dataLink property for controls, and the record buffering
					of the rowset object. Some methods and events
					for rowsets should work fine, however -- navigation,
					for example. 
					
					These buttons are part of the application framework
					designed for the book (mentioned above), and rely
					heavily on that framework to function properly.
					
					The Navigation Buttons will rely on the use of the 
					UILayer custom form's onNavigate() event handler to 
					re-load the data controls with the data from the table. 
					
					The form is always in an "edit" mode, so we don't need
					to worry about that functionality -- we cannot really
					rely on the rowset's state property to do anything.
					
					We will be relying on code in the custom form class
					UILayer.cfm, and the Data Module DataLayer.dmd to
					do some of what is needed.
	----------------------------------------------------------------------------				
*/

// See comments below regarding translations ...:
#include <custButtEnglish.h>    // completed
//#include <custButtItalian.h>    // completed
//#include <custButtGerman.h>     // completed
//#include <custButtDanish.h>     // completed
//#include <custButtDutch.h>      // completed
//#include <custButtNorwegian.h>  // completed
//#include <custButtFrench.h>     // completed
//#include <custButtCatalan.h>    // completed
//#include <custButtSpanish.h>    // completed
//#include <custButtPortuguese.h> // completed
//#include <custButtTurkish.h>    // completed
//#include <custButtSwedish.h>    // completed

#DEFINE ButtonWidth 64
#DEFINE ButtonHeight 24

/*
   To start, these buttons will be text-based,
	in a later chapter we will change them to 
	use images from the GlyphX image pack that
	is installed with dBASE ...
*/
class BaseButton( oParent ) of Pushbutton( oParent )
   with( this )
      text     := "Base Class"
		metric   := 6 // Pixels
		width    := ButtonWidth
		height   := ButtonHeight
		speedBar := true // can't get focus
   endwith
	// Custom Property:
	this.rowset = null // empty

   function getRowset()
      local rRowToActOn
      rRowToActOn = null // default
      // check to see if rowset property has been set -- allow
		// developer to reference a *specific* rowset rather than
		// the default form.rowset:
      if this.rowset # null
         rRowToActOn = this.rowset
        // ok, this.rowset has not been set, check to see
        // if the parent has a rowset that's valid
        // (the parent will be "NavigateButtons", a container,
        // or the form):
      elseif type( "this.parent.rowset" ) == "O" and;
             this.parent.rowset # null
         rRowToActOn = this.parent.rowset
      // otherwise we check for the form's rowset:
      elseif form.rowset # null
         rRowToActOn = form.rowset
      endif // this.rowset # null
   return rRowToActOn
endclass
// End of class: BaseButton

/* 
   Navigation Buttons:
	TopButton      -- First row of the rowset
	PreviousButton -- Back up one row from current
	NextButton     -- Move to the next row from current
	BottomButton   -- Last row of the rowset
*/
class TopButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "First"
   endwith

   function onClick
	   local rRowToActOn, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( NAVNOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( SAVECHANGES, ;
							  DATACHANGED,32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// navigate to first record
         rRowToActOn.first()
      endif
	return
endclass

class PreviousButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Previous"
   endwith

   function onClick
      local rRowToActOn, nState, nAnswer
      rRowToActOn = null

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( NAVNOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( SAVECHANGES, ;
							  DATACHANGED,32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// attempt to navigate:
         if ( not rRowToActOn.next(-1) )
            msgbox( ATFIRSTROW, ;
                    CANTNAVIGATE, 64 )
            rRowToActOn.next() // back off end of rowset marker
         endif // navigate
      endif // rRowToActOn
	return
endclass

class NextButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Next"
   endwith

   function onClick
      local rRowToActOn, nState, nAnswer
      rRowToActOn = null

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( NAVNOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( SAVECHANGES, ;
							  DATACHANGED,32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// attempt to navigate:
         if ( not rRowToActOn.next() )
            msgbox( ATLASTROW, ;
                    CANTNAVIGATE, 64 )
            rRowToActOn.next(-1) // back off end of rowset marker
         endif // attempt to navigate
      endif // form.modified
	return
endclass

class BottomButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Last"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( NAVNOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			if form.modified
				if msgbox( SAVECHANGES, ;
							  DATACHANGED,32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
         // to last row in rowset:		
         rRowToActOn.last()
      endif
	return
endclass

/*
   Buttons that deal with editing of the data:
	SaveButton    -- Save changes
	AbandonButton -- Abandon changes
	AppendButton  -- Clear out controls on form to allow user
	                 to enter new data
	DeleteButton  -- Delete the current row, and optionally
	                 cascade delete rows in a child table that
						  use the value of the primary key as 
						  a foreign key in that table to link the data.
*/
class SaveButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Save"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( SAVENOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
				return // nothing to save
			endif
			// call SaveRecord() method
			if form.modified
            form.SaveRecord()
			endif // form.modified
      endif // we have a valid rowset
	return // end of method
endclass

class AbandonButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Abandon"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( ABANDONNOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
				return // nothing to save
			endif
			// call AbandonRecord() method
			if form.modified
            form.AbandonRecord()
			endif // form.modified
      endif
	return
endclass

class AppendButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Append"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( ADDNOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // form.modified property should be being set
			// when a control loses focus if you are using
			// the SQLFormControls classes:
			if type( "form.modified" ) # "L"
				form.modified = false
			endif
			// as the AppendRecord method clears out the controls,
			// check to see if the user wants to save any changes:
			if form.modified
				if msgbox( SAVECHANGES, ;
							  DATACHANGED, 32+4) == 6 // Yes
					form.SaveRecord()
				endif // msgbox()
			endif // form.modified
			// call AppendRecord() method
         form.AppendRecord()
      endif
	return
endclass

class DeleteButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Delete"
   endwith
	// necessary for determining if a cascade
	// delete will occur. When placing this 
	// button on the form, in the form's
	// onOpen event handler, or overridden
	// Open method, these values should
	// be set:
	this.cascade         = false
	this.childTable      = null
	this.foreignKeyField = null // link to parent table
	
   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
         msgbox( DELETENOROWSET,;
                 CANTDOIT, 16 )
         return
      else
		   // ask if they really want to delete the record:
		   if msgbox( DELETEYESNO,;
		              ALERT, 32+4 ) == 6 // Yes
   			// call DeleteRecord() method
            form.DeleteRecord( this.cascade, this.childTable, ;
				                   this.foreignKeyField )
			endif // msgbox()
      endif // row ...
	return
endclass

/*
   Close the form, but before you do -- make sure the data has
	been saved if needed ...
*/
class CloseButton( oParent ) of BaseButton( oParent ) custom
   with( this )
      text := "Close"
   endwith

   function onClick
	   local rRowToActOn, nState, nAnswer
		rRowToActOn = null   

      // check to see if rowset property has been set:
      rRowToActOn = super::GetRowset()

      // is there a valid rowset?
      if rRowToActOn == null
		   // if not, then just close the form
         form.close()
      endif
		
	   // before closing the form, check to see if the
		// record is being modified (includes
		// appending -- if the user slipped into
		// append mode, but hasn't changed anything,
		// the form's modified property will be
		// false) -- and if so, ask if they 
		// want to save changes before closing:
		if form.modified
			if msgbox( DATACHANGED, ;
                    SAVECHANGES, 32+4) == 6 // Yes
            form.SaveRecord()
			endif // msgbox()
		endif // form.modified
      // close the form:
		form.close()
	return
endclass

/*
   ----------------------------------------------------------------------------
   End of file: SQLCustomButtons.cc
   ----------------------------------------------------------------------------
*/